#include "fftc8.h"

/* a[0...4n-1]; w[n...2n-2]; n >= 2 */
void fftc8_unpass(register fftc8 *a,register double *w,register int n)
{
  register double s1, s2, s3, s4, s5, s6, s7, s8;
  register fftc8 *b;
  register int j;

  b = a + n + n;
  j = n + n - 2;

  s1 = b[0].im;
  s2 = b[n].re;
  s3 = b[0].im;
  s4 = b[n].re;
  s1 += b[n].im;
  s2 += b[0].re;
  s3 -= b[n].im;
  s4 -= b[0].re;
  s5 = a[0].re;
  s6 = a[0].im;
  s7 = a[n].re;
  s5 -= s2;
  s6 -= s1;
  s7 -= s3;
  b[0].re = s5;
  b[0].im = s6;
  b[n].re = s7;
  s5 = a[n].im;

  ++a;
  ++b;

  s5 -= s4;
  s4 += a[n - 1].im;
  s2 += a[0 - 1].re;
  s1 += a[0 - 1].im;
  s3 += a[n - 1].re;
  b[n - 1].im = s5;
  a[0 - 1].re = s2;
  a[0 - 1].im = s1;
  a[n - 1].re = s3;
  a[n - 1].im = s4;

  do {
    s1 = b[0].re;
    s2 = b[0].im;
    s7 = w[n];
    s3 = b[0].re;
    s8 = w[j];
    s1 *= s7;
    s4 = b[0].im;
    s2 *= s8;
    s5 = b[n].re;
    s3 *= s8;
    s1 += s2;
    s4 *= s7;
    s6 = b[n].im;
    s5 *= s7;
    s7 = b[n].re;
    s4 -= s3;
    s6 *= s8;
    s3 = b[n].im;
    s5 -= s6;
    s7 *= s8;
    s6 = s1;
    s1 += s5;
    s3 *= w[n];
    s5 -= s6;
    s2 = a[0].re;
    s7 += s3;
    s8 = s4;
    s2 -= s1;
    s4 += s7;
    s8 -= s7;
    b[0].re = s2;
    s3 = a[0].im;
    s1 += a[0].re;
    s6 = a[n].re;
    s3 -= s4;
    s4 += a[0].im;
    a[0].re = s1;
    s6 -= s8;
    s8 += a[n].re;
    s2 = a[n].im;
    b[0].im = s3;
    s2 -= s5;
    b[n].re = s6;
    s5 += a[n].im;
    b[n].im = s2;
    a[0].im = s4;
    a[n].re = s8;
    a[n].im = s5;

    ++a;
    ++b;
    ++w;
    j -= 2;

  } while (j);
}

void inline fftc8_un2(register fftc8 *a)
{
  register double r0 = a[0].re;
  register double i0 = a[0].im;
  register double t = r0;
  register double u = i0;
  r0 += a[1].re;
  t -= a[1].re;
  i0 += a[1].im;
  u -= a[1].im;
  a[0].re = r0;
  a[0].im = i0;
  a[1].re = t;
  a[1].im = u;
}

void inline fftc8_un4(register fftc8 *a)
{
  register double s1, s2, s3, s4, s5, s6, s7, s8;

  s1 = a[0].re;
  s2 = a[0].im;
  s6 = a[3].re;
  s1 += a[1].re;
  s2 += a[1].im;
  s8 = s6;
  s6 += a[2].re;
  s4 = s1;
  s5 = s2;
  s1 -= s6;
  s4 += s6;
  s3 = a[0].re;
  s6 = a[2].im;
  a[0].re = s4;
  s7 = s6;
  s6 += a[3].im;
  s4 = a[0].im;
  s3 -= a[1].re;
  s5 += s6;
  s2 -= s6;
  s4 -= a[1].im;
  a[0].im = s5;
  s5 = s3;
  s7 -= a[3].im;
  s8 -= a[2].re;
  s6 = s4;
  s3 -= s7;
  s4 -= s8;
  s5 += s7;
  s6 += s8;
  a[2].re = s1;
  a[2].im = s2;
  a[3].re = s3;
  a[3].im = s4;
  a[1].re = s5;
  a[1].im = s6;
}

static double roots[] = {
  0.9238795325112867561281831893967882868224
, 0.7071067811865475244008443621048490392848
, 0.3826834323650897717284599840303988667613
, 0.9807852804032304491261822361342390369739
, 0.9238795325112867561281831893967882868224
, 0.8314696123025452370787883776179057567385
, 0.7071067811865475244008443621048490392848
, 0.5555702330196022247428308139485328743749
, 0.3826834323650897717284599840303988667613
, 0.1950903220161282678482848684770222409276
} ;

FFTC8_UN(inline fftc8_un8,fftc8_un4,fftc8_un2,roots - 1,2)
FFTC8_UN(fftc8_un16,fftc8_un8,fftc8_un4,roots - 4,4)
FFTC8_UN(fftc8_un32,fftc8_un16,fftc8_un8,roots - 5,8)
